(* poly_compare.ml: demonstrate cost of polymorphic comparison at
   runtime. Examine assembly code and timing. Compare with int_compare.ml

   SAMPLE RUN
   > ocamlopt -g poly_compare.ml
   > a.out 5000
   cmp_poly on ints
   count: 12496306,  time: 0.2570 secs
*)

open Printf;;

let cmp_poly a b = a < b;;                    (* compare any two things, polymorphic '<' *)

let _ =
  let narg = Array.length Sys.argv in
  if narg < 2 then
    (printf "usage: %s <int>\n" Sys.argv.(0);
     exit 1);

  let len = int_of_string Sys.argv.(1) in     (* create an array of random integers *)
  let nums = Array.make len 0 in              
  Random.init 123456789;
  for i=0 to len-1 do
    nums.(i) <- Random.int 10000;             (* random number between 0 and 10,000 *)
  done;

  printf "cmp_poly on ints\n";                (* using cmp_poly... *)
  let count = ref 0 in                        (* time comparison between all numbers *)
  let start_time = Sys.time () in
  for i=0 to len-1 do
    for j=0 to len-1 do
      if cmp_poly nums.(i) nums.(j) then
        incr count;
    done;
  done;
  let stop_time = Sys.time () in
  let tot_time = stop_time -. start_time in
  printf "count: %d,  time: %.4f secs\n" !count tot_time;

;;  
      
